/*
 * creature.c
 *
 *  Stellt alle Services zur Verfuegung, die benoetigt werden, um eine Kreatur auf dem Spielfeld zu setzen
 *	oder diese zu veraendern
 *
 *  Created on: 06.08.2015
 *      Author: Jun-Patrick Raabe & Kolya Feierabend
 */
#include "creature.h"

#include <stdlib.h>
#include <stdbool.h>

#include "config.h"
#include "output.h"

Creature getDefaultCreature() {
	Creature defaultCreature;
	defaultCreature.age = 0;
	defaultCreature.isHunter = false;
	defaultCreature.isHuman = false;
	defaultCreature.isBlade = false;
	defaultCreature.isVampire = false;
	defaultCreature.alreadyMoved = false;
	defaultCreature.hungerRate = 0;
	defaultCreature.woman = false;

	return defaultCreature;
}

void setCreatureToDefault(Creature *creature){
	creature->age = 0;
	creature->isHunter = false;
	creature->isHuman = false;
	creature->isBlade = false;
	creature->isVampire = false;
	creature->alreadyMoved = false;
	creature->hungerRate = 0;
	creature->woman = false;
}

void setDefaultToHuman(Creature *creature, int age, bool woman, int simStep, CounterSumAtSimStep* counterSumAtSimStep) {
	creature->age = age;
	creature->isHuman = true;
	creature->alreadyMoved = false;
	creature->hungerRate = 0;
	if (woman == true) {
		creature->woman = true;
	}
	counterSumAtSimStep->humanCounter++;
	counterSumAtSimStep->emptyCounter--;
}

void setDefaultToVampire(Creature *creature, int hungerrate, int simStep,  CounterSumAtSimStep* counterSumAtSimStep) {
	creature->isVampire = true;
	creature->alreadyMoved = false;
	creature->hungerRate = hungerrate;
	counterSumAtSimStep->vampireCounter++;
	counterSumAtSimStep->emptyCounter--;
}

void setDefaultToHunter(Creature *creature, int age, bool woman, int simStep, CounterSumAtSimStep* counterSumAtSimStep){
	creature->age = age;
	creature->isHunter = true;
	creature->alreadyMoved = false;
	creature->hungerRate = 0;
	if (woman) {
		creature->woman = true;
	}
	counterSumAtSimStep->hunterCounter++;
	counterSumAtSimStep->emptyCounter--;
}

void setDefaultToBlade(Creature *creature, int simStep, CounterSumAtSimStep* counterSumAtSimStep) {
	creature->isBlade = true;
	creature->alreadyMoved = false;
	counterSumAtSimStep->bladeCounter++;
	counterSumAtSimStep->emptyCounter--;
}

bool CreatureNotDefault (Creature *creature) {
	return (creature->isBlade || creature->isHuman || creature->isHunter || creature->isVampire);
}

void HumanToVampire (Creature *creature, CounterSumAtSimStep *counterSumAtSimStep) {
	creature->isHuman = false;
	creature->isVampire = true;
	creature->alreadyMoved = false;
	creature->hungerRate = 0;
	counterSumAtSimStep->humanCounter--;
	counterSumAtSimStep->vampireCounter++;
	}

void HumanToHunter(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep) {
	creature->isHunter = true;
	creature->isHuman = false;
	counterSumAtSimStep->humanCounter--;
	counterSumAtSimStep->hunterCounter++;
}

void HunterToVampire (Creature *creature, CounterSumAtSimStep *counterSumAtSimStep) {
	creature->isHunter = false;
	creature->isVampire = true;
	creature->alreadyMoved = false;
	creature->hungerRate = 0;
	counterSumAtSimStep->vampireCounter++;
	counterSumAtSimStep->hunterCounter--;
}

void HunterToHuman(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep) {
	creature->isHunter = false;
	creature->isHuman = true;
	counterSumAtSimStep->humanCounter++;
	counterSumAtSimStep->hunterCounter--;
}

void killVampire(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep){
	setCreatureToDefault(creature);
	counterSumAtSimStep->vampireCounter--;
	counterSumAtSimStep->emptyCounter++;
}

void killHunter(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep){
	setCreatureToDefault(creature);
	counterSumAtSimStep->hunterCounter--;
	counterSumAtSimStep->emptyCounter++;
}

void killHuman(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep){
	setCreatureToDefault(creature);
	counterSumAtSimStep->humanCounter--;
	counterSumAtSimStep->emptyCounter++;
}

