/*
 * creature.h
 *
 *	Stellt alle Services zur Verfuegung, die benoetigt werden, um eine Kreatur auf dem Spielfeld zu setzen
 *	oder diese zu veraendern
 *
 *  Created on: 06.08.2015
 *      Author: Jun-Patrick Raabe & Kolya Feierabend
 */
#include "output.h"
#include <stdbool.h>

#ifndef CREATURE_H_
#define CREATURE_H_

	/*
	 * Die Struktur Creature enthaelt alle relevanten Eigenschaften zur
	 * Identifikation der einzelnen Kreaturarten
	 */
	typedef struct{
		int age;
		bool woman;
		bool isHunter;
		bool isHuman;
		bool isBlade;
		bool isVampire;
		bool alreadyMoved;
		int hungerRate;
	}Creature;

	/*
	 * Gibt eine Kreature im Default-Zustand zurueck
	 */
	Creature getDefaultCreature();

	/*
	 * Setzt eine Kreatur zurueck auf den Default-Zustand
	 *
	 * @param *creature - Kreatur auf einer bestimmten Position des Spielfeldes, die
	 * auf Default gesetzt werden soll
	 */
	void setCreatureToDefault(Creature *creature);

	/*
	 * Setzt ein Defaultfeld zum Menschen
	 *
	 * @param *creature - Kreatur auf einer bestimmten Position des Spielfeldes, die
	 * zum Menschen gesetzt werden soll
	 * @param age - das zufllige Alter, dass die Kreatur erhalten soll
	 * @param woman - das Geschlecht, dass die Kreatur erhalten soll
	 * @param simStep - der Simulationsschritt in dem die Veraenderung statt findet
	 */
	void setDefaultToHuman(Creature *creature, int age, bool woman, int simStep, CounterSumAtSimStep* counterSumAtSimStep);

	/*
	 * Setzt ein Defaultfeld zum Vampir
	 *
	 * @param *creature - Kreatur auf einer bestimmten Position des Spielfeldes, die
	 * zum Vampir gesetzt werden soll
	 * @param hungerrate - die Hungerrate, die die Kreatur erhalten soll
	 * @param simStep - der Simulationsschritt in dem die Veraenderung statt findet
	 */
	void setDefaultToVampire(Creature *creature, int hungerrate, int simStep, CounterSumAtSimStep* counterSumAtSimStep);

	/*
	 * Setzt ein Defaultfeld zum Jaeger
	 *
	 * @param *creature - Kreatur auf einer bestimmten Position des Spielfeldes, die
	 * zum Jaeger gesetzt werden soll
	 * @param age - das zufllige Alter, dass die Kreatur erhalten soll
	 * @param woman - das Geschlecht, dass die Kreatur erhalten soll
	 * @param simStep - der Simulationsschritt in dem die Veraenderung statt findet
	 */
	void setDefaultToHunter(Creature *creature, int age, bool woman, int simStep, CounterSumAtSimStep* counterSumAtSimStep);

	/*
	 * Setzt ein Defaultfeld zu Blade
	 *
	 * @param *creature - Kreatur auf einer bestimmten Position des Spielfeldes, die
	 * zu Blade gesetzt werden soll
	 * @param simStep - der Simulationsschritt in dem die Veraenderung statt findet
	 */
	void setDefaultToBlade(Creature *creature, int simStep, CounterSumAtSimStep* counterSumAtSimStep);

	/*
	 * Ueberprueft, ob ein Feld im Spielfeld auf Default gesetzt ist oder nicht.
	 *
	 * @param *creature - Kreatur auf einer bestimmten Position des Spielfeldes, die ueberprueft werden soll
	 */
	bool CreatureNotDefault (Creature *creature);

	/*
	 * Setzt einen Menschen zum Vampir
	 *
	 * @param *creature - Mensch auf einer bestimmten Position des Spielfeldes, die
	 * zum  Vampir werden soll
	 * @param *counterSumAtSimStep - alle Counter zu dem bestimmten Zeitpunkt
	 * des Simulationsschritts
	 */
	void HumanToVampire (Creature *creature, CounterSumAtSimStep *counterSumAtSimStep);

	/*
	 * Setzt einen Menschen zum Jaeger
	 *
	 * @param *creature - Mensch auf einer bestimmten Position des Spielfeldes, die
	 * zum  Jaeger werden soll
	 * @param *counterSumAtSimStep - alle Counter zu dem bestimmten Zeitpunkt
	 * des Simulationsschritts
	 */
	void HumanToHunter(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep);

	/*
	 * Setzt einen Jaeger zum Vampir
	 *
	 * @param *creature - Jaeger auf einer bestimmten Position des Spielfeldes, die
	 * zum  Vampir werden soll
	 * @param *counterSumAtSimStep - alle Counter zu dem bestimmten Zeitpunkt
	 * des Simulationsschritts
	 */
	void HunterToVampire (Creature *creature, CounterSumAtSimStep *counterSumAtSimStep);

	/*
	 * Setzt einen Jaeger zum Menschen
	 *
	 * @param *creature - Jaeger auf einer bestimmten Position des Spielfeldes, die
	 * zum Menschen werden soll
	 * @param *counterSumAtSimStep - alle Counter zu dem bestimmten Zeitpunkt
	 * des Simulationsschritts
	 */
	void HunterToHuman(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep);

	/*
	 * Setzt einen Vampir zurueck auf Default
	 *
	 * @param *creature - Vampir auf einer bestimmten Position des Spielfeldes, der
	 * zu Default gesetzt werden soll
	 * @param *counterSumAtSimStep - alle Counter zu dem bestimmten Zeitpunkt
	 * des Simulationsschritts
	 */
	void killVampire(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep);

	/*
	 * Setzt einen Jaeger zurueck auf Default
	 *
	 * @param *creature - Jaeger auf einer bestimmten Position des Spielfeldes, der
	 * zu Default gesetzt werden soll
	 * @param *counterSumAtSimStep - alle Counter zu dem bestimmten Zeitpunkt
	 * des Simulationsschritts
	 */
	void killHunter(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep);

	/*
	 * Setzt einen Menschen zurueck auf Default
	 *
	 * @param *creature - Mensch auf einer bestimmten Position des Spielfeldes, der
	 * zu Default gesetzt werden soll
	 * @param *counterSumAtSimStep - alle Counter zu dem bestimmten Zeitpunkt
	 * des Simulationsschritts
	 */
	void killHuman(Creature *creature, CounterSumAtSimStep *counterSumAtSimStep);
#endif /* CREATURE_H_ */
