/*
 * gamefield.c
 *
 * Stellt alle Services zur Verfuegung, die benoetigt werden um die Simulation
 * auszufuehren.
 *
 *  Created on: 10.08.2015
 *      Author: Jun-Patrick Raabe & Kolya Feierabend
 */
#include "gamefield.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <time.h>

#include "creature.h"
#include "config.h"
#include "generateandset.h"

Creature *gamefield;
int X_SIZE, Y_SIZE, Z_SIZE;

void generateAndSetDefault();
void applyHumanRules(Creature *human, int sim_step);
void applyHunterRules(Creature *hunter, int sim_step);
void applyVampireRules(Creature *vampire, int sim_step);
void fightHumanVampire(Creature *mensch, Creature *vampire, int sim_step);
void CheckForFightHumanVampire(int x, int y, int sim_step);
void fightVampireHunter(Creature *vampire, Creature *hunter, int sim_step);
void CheckForFightVampireHunter(int x, int y, int sim_step);
void CheckForFightBladeVampire(int x, int y, int sim_step);
void moveHuman(int x, int y, int z);
void moveVampireOrHunterOrBlade (int x, int y, int z);
void randomMove(int old_x_pos, int old_y_pos, int old_z_pos);
void escapeVampires(int x, int y, int z, int Danger[3][3], int maxDangerCounter);
void huntVictim(int x, int y, int z, int TargetCounter[3][3]);
int getNeighbourhoodSize(int x, int y);



void initGamefield(){
	X_SIZE = world_size_x + 1;
	Y_SIZE = world_size_y + 1;
	Z_SIZE = 2;

	gamefield = (Creature *)malloc(sizeof(Creature) * X_SIZE*Y_SIZE*Z_SIZE);

	generateAndSetDefault();
}


int GetNextXPos (int x) {
	int verschiebung_x;
	if (x == 0)
	{
		verschiebung_x = rand() % 2;
	} else if (x == world_size_x) {
		verschiebung_x = (rand() % 2) - 1;
	} else {
		verschiebung_x = (rand() % 3) - 1;
	}
	return verschiebung_x + x;
}

int GetNextYPos (int y) {
	int verschiebung_y;
	if (y == 0) {
		verschiebung_y = rand() % 2;
	} else if (y == world_size_y) {
		verschiebung_y = (rand() % 2) - 1;
	} else {
		verschiebung_y = (rand() % 3) - 1;
	}
	return verschiebung_y + y;
}

void useAllRules(int sim_step, CounterSumAtSimStep* counterSumAtSimStep){
	for(int x = allowed_min_x_Pos; x <= allowed_max_x_Pos; ++x) {
		for (int y = 0; y <= world_size_y ; ++y) {
			for (int z = 0; z < 2; ++z) {
				if(CreatureNotDefault((&gamefield[getPositionInArray(x, y, z)])) && gamefield[getPositionInArray(x, y, z)].alreadyMoved) {
					//Kampf Mensch Vampir
					CheckForFightHumanVampire(x, y, sim_step);

					// Kampf Vampir Hunter
					CheckForFightVampireHunter(x, y, sim_step);

					//Kampf Blade Vampire
					CheckForFightBladeVampire(x, y, sim_step);

					//Menschen Regelwerk anwenden
					applyHumanRules(&gamefield[getPositionInArray(x, y, z)], sim_step);

					//Hunter Regelwerk anwenden
					applyHunterRules(&gamefield[getPositionInArray(x, y, z)], sim_step);

					//Vampire Regelwerk anwenden
					applyVampireRules(&gamefield[getPositionInArray(x, y, z)], sim_step);

					gamefield[getPositionInArray(x, y, z)].alreadyMoved = false;
				}
			}
		}
	}
	MPI_Barrier(MPI_COMM_WORLD);
	if (rank != 0) {
		//receive left boarder
		MPI_Recv(&gamefield[getPositionInArray((allowed_min_x_Pos - 2), 0, 0)], (world_size_y + 1) * 2 * 2, mpi_creature, (rank-1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	}
	if (rank != (numprocs - 1)) {
		//send right boarder
		MPI_Send(&gamefield[getPositionInArray((allowed_max_x_Pos - 1), 0, 0)], (world_size_y + 1) * 2 * 2, mpi_creature, (rank + 1), (rank + 1), MPI_COMM_WORLD);
	}
	if (rank != (numprocs - 1)) {
		//receive right boarder
		MPI_Recv(&gamefield[getPositionInArray((allowed_max_x_Pos + 1), 0, 0)], (world_size_y + 1) * 2 * 2, mpi_creature, (rank+1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	}
	if (rank != 0) {
		//send left boarder
		MPI_Send(&gamefield[getPositionInArray(allowed_min_x_Pos, 0, 0)], (world_size_y + 1) * 2 * 2, mpi_creature, (rank - 1), (rank - 1), MPI_COMM_WORLD);
	}
}

bool KeinPlatzInUmgebung(int x, int y) {
	for (int a = -1; a < 2; ++a) {
		for (int b = -1; b < 2 ; ++b) {
			int test_x = x + a;
			int test_y = y + b;
			if ((test_x >= 0) && (test_x <= world_size_x) && (test_y >= 0) && (test_y <= world_size_y)) {
				for (int z = 0; z < 2; ++z) {
					if (!CreatureNotDefault((&gamefield[getPositionInArray(test_x, test_y, z)]))) {
						return false;
					}
				}
			}
		}
	}
	return true;
}

void moveCreaturesAndSetNewborns(int sim_step) {
	if ((rank % 2) == 1) {
		//receive left boarder area
		MPI_Recv(&gamefield[getPositionInArray((allowed_min_x_Pos - 2), 0, 0)], (world_size_y + 1) * 2 * 4, mpi_creature, (rank-1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
		if (rank != (numprocs - 1)) {
			//receive right boarder area
			MPI_Recv(&gamefield[getPositionInArray((allowed_max_x_Pos - 1), 0, 0)], (world_size_y + 1) * 2 * 4, mpi_creature, (rank+1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
		}
	}
	for(int x = allowed_min_x_Pos; x <= allowed_max_x_Pos; x++)
	{
		for (int y = 0; y <= world_size_y ; ++y)
		{
			for (int z = 0; z < 2; ++z)
			{
				if (z == 0) {
					//Newborn
					setNewbornIfPossibile(x, y, sim_step, &counterSum[sim_step]);
				}

				if (!(gamefield[getPositionInArray(x, y, z)].alreadyMoved)){
					if (CreatureNotDefault((&gamefield[getPositionInArray(x, y, z)])))
					{
						if(KeinPlatzInUmgebung(x, y)) {
							gamefield[getPositionInArray(x, y, z)].alreadyMoved = true;
						//Menschen-Bewegung
						} else if(gamefield[getPositionInArray(x, y, z)].isHuman) {
							moveHuman(x, y, z);
						}
						//Vampire-Bewegung
						else if(gamefield[getPositionInArray(x, y, z)].isVampire){
							moveVampireOrHunterOrBlade(x, y, z);
						}
						//Hunter-/Blade-Bewegung
						else if(gamefield[getPositionInArray(x, y, z)].isHunter || gamefield[getPositionInArray(x, y, z)].isBlade){
							moveVampireOrHunterOrBlade(x, y, z);
						}
					}
				}
			}
		}
	}
	if (rank != 0) {
		//send left boarder area
		MPI_Send(&gamefield[getPositionInArray((allowed_min_x_Pos - 2), 0, 0)], (world_size_y + 1) * 2 * 4, mpi_creature, (rank - 1), (rank - 1), MPI_COMM_WORLD);
	}
	if (rank != (numprocs - 1)) {
		//MPI_Send rechte Nachbar-Spalten
		MPI_Send(&gamefield[getPositionInArray((allowed_max_x_Pos - 1), 0, 0)], (world_size_y + 1) * 2 * 4, mpi_creature, (rank + 1), (rank + 1), MPI_COMM_WORLD);
	}
	if ((rank % 2) == 0) {
		if (rank != 0) {
			//receive left boarder area
			MPI_Recv(&gamefield[getPositionInArray((allowed_min_x_Pos - 2), 0, 0)], (world_size_y + 1) * 2 * 4, mpi_creature, (rank-1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
		}
		if (rank != (numprocs - 1)) {
			//receive right boarder area
			MPI_Recv(&gamefield[getPositionInArray((allowed_max_x_Pos - 1), 0, 0)], (world_size_y + 1) * 2 * 4, mpi_creature, (rank+1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
		}
	}
}

/*
 * Setzt alle Kreaturen auf dem Spielfeld auf Default und erhoeht den Counter entsprechend.
 */
void generateAndSetDefault(){
	for(int other_x = 0; other_x <= world_size_x ; ++other_x) {
		for (int other_y = 0; other_y <= world_size_y ; ++other_y){
			for (int other_z = 0; other_z < 2; ++other_z) {
				gamefield[getPositionInArray(other_x, other_y, other_z)] = getDefaultCreature();
				counterSum[0].emptyCounter++;
			}
		}
	}
}


/*
 * Wendet alle Regeln an die fuer einen Menschen von Bedeutung sind (Sterben im gewissen Alter
 * und transformieren des Menschens zum Hunter im Kampfalter).
 *
 * @param *human - der Mensch auf den die Regeln angewendet werden sollen
 * @param sim_step - der aktuelle Simulationsschritt
 */
void applyHumanRules(Creature *human, int sim_step){
	if (human->isHuman) {
		human->age++;
		if (human->age >= human_death_age + ((rand() % 21) - 10)){
			killHuman(human, &counterSum[sim_step]);
		}else if ((human->age >= min_fighting_age) && (human->age <= max_fighting_age)) {
			HumanToHunter(human, &counterSum[sim_step]);
		}
	}
}

/*
 * Wendet alle Regeln an die fuer einen Menschen von Bedeutung sind (transformieren des Jaegers
 * zum Menschen wenn er nicht mehr im Kampfalter ist).
 *
 * @param *hunter - der Jaeger auf den die Regeln angewendet werden sollen
 * @param sim_step - der aktuelle Simulationsschritt
 */
void applyHunterRules(Creature *hunter, int sim_step){
	if (hunter->isHunter) {
		hunter->age++;
		if (hunter->age > max_fighting_age) {
			HunterToHuman(hunter, &counterSum[sim_step]);
		}
	}
}

/*
 * Wendet alle Regeln an die fuer einen Vampir von Bedeutung sind (verhungern lassen des Vampirs,
 * wenn die Hungerrate zu hoch ist)
 *
 * @param *vampire - der Vampir auf den die Regeln angewendet werden sollen
 * @param sim_step - der aktuelle Simulationsschritt
 */
void applyVampireRules(Creature *vampire, int sim_step){
	if (vampire->isVampire) {
		vampire->hungerRate++;
		//verhungert
		if (vampire->hungerRate > vampire_max_hunger_rate) {
			killVampire(vampire, &counterSum[sim_step]);
		}
	}
}

/*
 * Fuehrt einen Kampf zwischen einem Vampir und einem Menschen aus mit all seinen Folgen.
 *
 *  @param *mensch - der im Kampf beteiligte Mensch
 *  @param *vampire - der im Kampf beteiligte Vampir
 *  @param sim_step - der aktuelle Simulationsschritt
 */
void fightHumanVampire(Creature *mensch, Creature *vampire, int sim_step){
	int dead_or_vampire;
	dead_or_vampire = (rand() % 100) + 1;

	if (dead_or_vampire >= human_to_vampire_percentage) {
		//toete Mensch
		killHuman(mensch, &counterSum[sim_step]);
	} else {
		//Mensch wird Vampir
		HumanToVampire(mensch, &counterSum[sim_step]);
	}
	//saettige Vampir
	vampire->hungerRate = 0;
	vampire->alreadyMoved = false;
	mensch->alreadyMoved = false;
}


/*
 * Prueft ob es auf der Position einen Kampf zwischen einem Vampir und einem Menschen
 * geben kann und leitet diesen ggf. ein.
 *
 * @param x - die zu pruefende X-Koordinate der Position
 * @param y - die zu pruefende Y-Koordinate der Position
 * @param sim_step - der aktuelle Simulationsschritt
 */
void CheckForFightHumanVampire(int x, int y, int sim_step){
	if ((gamefield[getPositionInArray(x, y, 0)].isVampire) && (gamefield[getPositionInArray(x, y, 1)].isHuman)) {
		fightHumanVampire(&gamefield[getPositionInArray(x, y, 1)], &gamefield[getPositionInArray(x, y, 0)], sim_step);

	} else if ((gamefield[getPositionInArray(x, y, 0)].isHuman) && (gamefield[getPositionInArray(x, y, 1)].isVampire)) {
		fightHumanVampire(&gamefield[getPositionInArray(x, y, 0)], &gamefield[getPositionInArray(x, y, 1)], sim_step);

	}

}

/*
 * Fuehrt einen Kampf zwischen einem Vampir und einem Jaeger aus mit all seinen Folgen.
 *
 *  @param *vampire - der im Kampf beteiligte Vampir
 *  @param *hunter - der im Kampf beteiligte Jaeger
 *  @param sim_step - der aktuelle Simulationsschritt
 */
void fightVampireHunter(Creature *vampire, Creature *hunter, int sim_step){
	int hunter_win;
	hunter_win = (rand() % 100) + 1;
	if (hunter_win <= hunter_win_percentage) {
		//Hunter wins - toete Vampir
		killVampire(vampire, &counterSum[sim_step]);
	} else {
		//Vampire wins
		int dead_or_vampire;
		dead_or_vampire = (rand() % 100) + 1;

		if (dead_or_vampire >= human_to_vampire_percentage) {
			//toete Hunter
			killHunter(hunter, &counterSum[sim_step]);
		} else {
			//Hunter wird Vampir
			HunterToVampire(hunter, &counterSum[sim_step]);
		}
		//saettige Vampir
		vampire->hungerRate = 0;
	}
	vampire->alreadyMoved = false;
	hunter->alreadyMoved = false;
}

/*
 * Prueft ob es auf der Position einen Kampf zwischen einem Vampir und einem Jaeger
 * geben kann und leitet diesen ggf. ein.
 *
 * @param x - die zu pruefende X-Koordinate der Position
 * @param y - die zu pruefende Y-Koordinate der Position
 * @param sim_step - der aktuelle Simulationsschritt
 */
void CheckForFightVampireHunter(int x, int y, int sim_step){
	if (gamefield[getPositionInArray(x, y, 0)].isVampire && gamefield[getPositionInArray(x, y, 1)].isHunter) {
		fightVampireHunter(&gamefield[getPositionInArray(x, y, 0)], &gamefield[getPositionInArray(x, y, 1)], sim_step);
		gamefield[getPositionInArray(x, y, 1)].age++;
	} else if (gamefield[getPositionInArray(x, y, 0)].isHunter && gamefield[getPositionInArray(x, y, 1)].isVampire) {
		fightVampireHunter(&gamefield[getPositionInArray(x, y, 1)], &gamefield[getPositionInArray(x, y, 0)], sim_step);
	}
}

/*
 * Prueft ob es auf der Position einen Kampf zwischen einem Vampir und Blade
 * geben kann und fuehrt diesen ggf. durch.
 *
 * @param x - die zu pruefende X-Koordinate der Position
 * @param y - die zu pruefende Y-Koordinate der Position
 * @param sim_step - der aktuelle Simulationsschritt
 */
void CheckForFightBladeVampire(int x, int y, int sim_step){
	if ((gamefield[getPositionInArray(x, y, 0)].isVampire && gamefield[getPositionInArray(x, y, 1)].isBlade)) {
		killVampire(&gamefield[getPositionInArray(x, y, 0)], &counterSum[sim_step]);
		gamefield[getPositionInArray(x, y, 0)].alreadyMoved = false;
		gamefield[getPositionInArray(x, y, 1)].alreadyMoved = false;
	} else if ((gamefield[getPositionInArray(x, y, 0)].isBlade && gamefield[getPositionInArray(x, y, 1)].isVampire)) {
		killVampire(&gamefield[getPositionInArray(x, y, 1)], &counterSum[sim_step]);
		gamefield[getPositionInArray(x, y, 0)].alreadyMoved = false;
		gamefield[getPositionInArray(x, y, 1)].alreadyMoved = false;
	}
}

/*
 * Bewegt die Kreatur auf dem Platz vom Alten auf den Neuen
 *
 * @param old_x - die X-Koordinate der alten Position
 * @param old_y - die Y-Koordinate der alten Position
 * @param old_z - die Z-Koordinate der alten Position
 * @param new_x - die X-Koordinate der neuen Position
 * @param new_y - die Y-Koordinate der neuen Position
 * @param new_z - die Z-Koordinate der neuen Position
 */
void moveCreature(int old_x, int old_y, int old_z, int new_x, int new_y, int new_z) {
	//Creature auf den neuen Platz setzen
	gamefield[getPositionInArray(new_x, new_y, new_z)].age = gamefield[getPositionInArray(old_x, old_y, old_z)].age;
	gamefield[getPositionInArray(new_x, new_y, new_z)].woman = gamefield[getPositionInArray(old_x, old_y, old_z)].woman;
	gamefield[getPositionInArray(new_x, new_y, new_z)].isBlade = gamefield[getPositionInArray(old_x, old_y, old_z)].isBlade;
	gamefield[getPositionInArray(new_x, new_y, new_z)].isHuman = gamefield[getPositionInArray(old_x, old_y, old_z)].isHuman;
	gamefield[getPositionInArray(new_x, new_y, new_z)].isHunter = gamefield[getPositionInArray(old_x, old_y, old_z)].isHunter;
	gamefield[getPositionInArray(new_x, new_y, new_z)].isVampire = gamefield[getPositionInArray(old_x, old_y, old_z)].isVampire;
	gamefield[getPositionInArray(new_x, new_y, new_z)].hungerRate = gamefield[getPositionInArray(old_x, old_y, old_z)].hungerRate;
	gamefield[getPositionInArray(new_x, new_y, new_z)].alreadyMoved = true;

	//Creature auf dem alten Platz lschen
	setCreatureToDefault(&gamefield[getPositionInArray(old_x, old_y, old_z)]);
}

/*
 * Gibt einen Wert zwischen 0..17 zurueck, der die Hoehe der Gefahr fuer das Auftreten eines Vampirs
 * ausdrueckt.
 *
 * @param x - die X-Koordinate der Stelle in dessen Umkreis gesucht werden soll
 * @param y - die Y-Koordinate der Stelle in dessen Umkreis gesucht werden soll
 */
int vampireDanger(int x, int y) {
	int vampireDangerCounter = 0;
	int test_x;
	int test_y;
	for(int a = -1; a < 2; ++a) {
		for(int b = - 1; b < 2; ++b) {
			test_x = x + a;
			test_y = y + b;
			if ((test_x >= 0) && (test_x <= world_size_x) && (test_y >= 0) && (test_y <= world_size_y)) {
				for (int z = 0; z < 2; ++z) {
					if(gamefield[getPositionInArray(test_x, test_y, z)].isVampire) {
						vampireDangerCounter++;
					}
				}
			}
		}
	}
	return vampireDangerCounter;
}

/*
 * Gibt einen Wert zwischen 0..17 zurueck, der die Chance ein Opfer fuer ein Vampir bzw. einen
 * Vampirjaeger zu finden ausdrueckt.
 *
 * @param x - die X-Koordinate der Stelle in dessen Umkreis gesucht werden soll
 * @param y - die Y-Koordinate der Stelle in dessen Umkreis gesucht werden soll
 */
int GetTargetCounter(int x, int y, int xpos_raider, int ypos_raider, int zpos_raider){
	int targetCounter = 0;
	int test_x;
	int test_y;
	for(int a = -1; a < 2; ++a) {
		for(int b = - 1; b < 2; ++b) {
			test_x = x + a;
			test_y = y + b;
			if ((test_x >= 0) && (test_x <= world_size_x) && (test_y >= 0) && (test_y <= world_size_y)) {
				for (int z = 0; z < 2; ++z) {
					if(gamefield[getPositionInArray(xpos_raider, ypos_raider, zpos_raider)].isVampire){
						if(gamefield[getPositionInArray(test_x, test_y, z)].isHuman) {
							targetCounter++;
						}
					}else if(gamefield[getPositionInArray(xpos_raider, ypos_raider, zpos_raider)].isHunter || gamefield[getPositionInArray(xpos_raider, ypos_raider, zpos_raider)].isBlade ){
						if(gamefield[getPositionInArray(test_x, test_y, z)].isVampire) {
							targetCounter++;
						}
					}
				}
			}
		}
	}
	return targetCounter;
}

/*
 * Bewegt einen Menschen von der Stelle x, y, z weg und laesst ihn dabei von Vampiren weglaufen.
 *
 * @param x - die aktuelle X-Koordinate der Position des Menschens
 * @param y - die aktuelle Y-Koordinate der Position des Menschens
 * @param z - die aktuelle Z-Koordinate der Position des Menschens
 *
 */

void moveHuman(int x, int y, int z) {
	int maxDangerCounter = 0;
	int Danger[3][3];
	int test_x, test_y;
	for(int a = -1; a < 2; ++a) {
		for (int b = -1; b < 2; ++b) {
			test_x = x+a;
			test_y = y+b;
			if ((test_x >= 0) && (test_x <= world_size_x) && (test_y >= 0) && (test_y <= world_size_y)) {
				Danger[a+1][b+1] = vampireDanger(x+a, y+b);
				if(maxDangerCounter < vampireDanger(x+a,y+b)){
					maxDangerCounter = vampireDanger(x+a,y+b);
				}
			} else {
				Danger[a+1][b+1] = -1;
			}
		}
	}

	if(maxDangerCounter > 0){
		escapeVampires(x, y, z, Danger, maxDangerCounter);
	}else{
		randomMove(x, y, z);
	}
}

/*
 * Bewegt einen Vampir, einen Jaeger oder Blade von der Stelle x, y, z weg und laesst ihn dabei
 * seine Opfer jagen.
 *
 * @param x - die aktuelle X-Koordinate der Position des Menschen
 * @param y - die aktuelle Y-Koordinate der Position des Menschen
 * @param z - die aktuelle Z-Koordinate der Position des Menschen
 *
 */
void moveVampireOrHunterOrBlade (int x, int y, int z) {
	int maxTargetCounter = 0;
	int TargetCounter[3][3];
	int test_x, test_y;
	//Suche nach Opfern
	for(int a = -1; a < 2; ++a) {
		for (int b = -1; b < 2; ++b) {
			test_x = x+a;
			test_y = y+b;
			if ((test_x >= 0) && (test_x <= world_size_x) && (test_y >= 0) && (test_y <= world_size_y)) {
				TargetCounter[a+1][b+1] = GetTargetCounter(x+a, y+b, x, y, z);
				if(maxTargetCounter < GetTargetCounter(x+a, y+b, x, y, z)){
					maxTargetCounter = GetTargetCounter(x+a, y+b, x, y, z);
				}
			} else {
				TargetCounter[a+1][b+1] = -1;
			}
		}
	}
	if(maxTargetCounter > 0){
		huntVictim(x, y, z, TargetCounter);

	}else{
		randomMove(x,y,z);
	}
}


/*
 * Bewegt eine Kreatur zufllig
 *
 * @param old_x_pos - X Position
 * @param old_y_pos - Y Position
 * @param old_z_pos - Z Position
 */
void randomMove(int old_x_pos, int old_y_pos, int old_z_pos){
	int new_x_pos, new_y_pos, new_z_pos;
	bool creatureMoved = false;
	do{
		new_x_pos = GetNextXPos(old_x_pos);
		new_y_pos = GetNextYPos(old_y_pos);
		for (new_z_pos = 0; new_z_pos < 2; ++new_z_pos){
			if(!CreatureNotDefault((&gamefield[getPositionInArray(new_x_pos, new_y_pos, new_z_pos)]))){
				moveCreature(old_x_pos, old_y_pos, old_z_pos, new_x_pos, new_y_pos, new_z_pos);
				creatureMoved = true;
				break;
			}
		}
	} while (!creatureMoved);
}

/*
 * Sucht nach Vampiren und bewegt sich weg von ihnen
 *
 * @param x - X Position
 * @param y - Y Position
 * @param z - Z Position
 * @param Danger[3][3] - Wert zwischen 0..17 in der jeweiligen Position, der die Chance ausdrckt, nicht gefressen zu werden.
 */
void escapeVampires(int x, int y, int z, int Danger[3][3], int maxDangerCounter){
	bool done = false;
	int posChecked[3][3];
	int a, b, checkedPosCounter;

	for (int danger = 0; danger <= maxDangerCounter ; ++danger) {
		if (done) {
			break;
		}
		for(int i = 0; i < 3; ++i){
			for(int j = 0; j < 3; ++j){
				posChecked[i][j] = 0;
			}
		}
		checkedPosCounter = 0;

		do{
			a = GetNextXPos(x)-x;
			b = GetNextYPos(y)-y;
			if (posChecked[a+1][b+1] == 0)
			{
				if(Danger[a+1][b+1] == danger) {
					int new_x_pos = x + a;
					int new_y_pos = y + b;
					for(int place = 0; place < 2; ++place) {
						if (!CreatureNotDefault((&gamefield[getPositionInArray(new_x_pos, new_y_pos, place)]))) {
							moveCreature(x, y, z, new_x_pos, new_y_pos, place);
							done = true;
							break;
						}
					}
				}
				posChecked[a+1][b+1] = 1;
				checkedPosCounter++;
			}
		} while((!done) && (checkedPosCounter <= getNeighbourhoodSize(x, y)));

	}
}


/*
 * Sucht nach Opfer-Kreaturen und bewegt sich in deren Richtung
 *
 * @param x - X Position
 * @param y - Y Position
 * @param z - Z Position
 * @param TargetCounter[3][3] - Wert zwischen 0..17 in der jeweiligen Position, der die Chance ein Opfer fuer ein Vampir bzw. einen
 * Vampirjaeger zu finden ausdrueckt.
 */
void huntVictim(int x, int y, int z, int TargetCounter[3][3]){
	bool done = false;
	int posChecked[3][3];
	int a, b, checkedPosCounter;

	for (int targets = 18; targets >= 0 ; --targets) {
		if (done == true) {
			break;
		}
		for(int i = 0; i < 3; ++i){
			for(int j = 0; j < 3; ++j){
				posChecked[i][j] = 0;
			}
		}
		checkedPosCounter = 0;
		do{
			a = GetNextXPos(x)-x;
			b = GetNextYPos(y)-y;
			if (posChecked[a+1][b+1] == 0)
				{
					if(TargetCounter[a+1][b+1] == targets) {
						int new_x_pos = x + a;
						int new_y_pos = y + b;
						for(int place = 0; place < 2; ++place) {
							if (!CreatureNotDefault((&gamefield[getPositionInArray(new_x_pos, new_y_pos, place)]))) {
								moveCreature(x, y, z, new_x_pos, new_y_pos, place);
								done = true;
								break;
							}
						}
					}
					posChecked[a+1][b+1] = 1;
					checkedPosCounter++;
				}
		}while((!done) && (checkedPosCounter <= getNeighbourhoodSize(x, y)));
	}
}


/*
 * Gibt die Anzahl der Nachbarfelder zurck. Es kann die Werte 3, 5 und 8 annehmen.
 *
 * @param x - X Position
 * @param y - Y Position
 */
int getNeighbourhoodSize(int x, int y){
	int neighbourhoodSize = 8;
	if(x == 0){
		neighbourhoodSize = neighbourhoodSize - 3;
	}else if(x == world_size_x){
		neighbourhoodSize = neighbourhoodSize - 3;
	}
	if(y == 0){
		neighbourhoodSize = neighbourhoodSize - 3;
	}else if(y == world_size_y){
		neighbourhoodSize = neighbourhoodSize - 3;
	}

	if (((x == 0) && (y == 0)) || ((x == 0) && (y == world_size_y)) || ((x == world_size_x) && (y == 0)) || ((x == world_size_x) && (y == world_size_y)) ) {
		neighbourhoodSize = neighbourhoodSize + 1;
	}

	return neighbourhoodSize;
}

/*
 * Gibt die Position im Gamefield-Array zurck
 *
 * @param x - X Position
 * @param y - Y Position
 * @param z - Z Position
 */
int getPositionInArray(int x, int y, int z) {
	return x * Y_SIZE * Z_SIZE + y * Z_SIZE + z;
}



