/*
 * gamefield.h
 *
 * Stellt alle Services zur Verfuegung, die benoetigt werden um die Simulation
 * auszufuehren.
 *
 *  Created on: 10.08.2015
 *      Author: Jun-Patrick Raabe & Kolya Feierabend
 */
#include "creature.h"
#include <stdbool.h>

#ifndef GAMEFIELD_H_
#define GAMEFIELD_H_

	/*
	 * Stellt das Spielfeld ueberall zur Verfuegung um Regeln und Bewegungen anzuwenden.
	 */
	extern Creature *gamefield;

	/*
	 * Gibt die konktrete Groee des Spielfelds in all seinen Dimensionen an.
	 */
	extern int X_SIZE, Y_SIZE, Z_SIZE;

	/*
	 * Initialisiert die Area des jeweiligen Prozesses und befuellt es mit Default Creatures.
	 */
	void initGamefield();

	/*
	 * Gibt eine neue gueltige X-Position zurueck, die im Umkreis der aktuellen X-Pos liegt
	 * und dabei beruecksichtigt, wenn diese Position am linken oder rechten Rand liegt.
	 *
	 * @param x - die akteuelle X-Postion von der aus geschaut werden soll
	 */
	int GetNextXPos (int x);

	/*
	 * Gibt eine neue gueltige Y-Position zurueck, die im Umkreis der aktuellen Y-Pos liegt
	 * und dabei beruecksichtigt, wenn diese Position am oberen oder unteren Rand liegt.
	 *
	 * @param y - die akteuelle Y-Postion von der aus geschaut werden soll
	 */
	int GetNextYPos (int y);

	/*
	 * Wendet alle Regel auf dem aktuellen Spielfeld an.
	 *
	 * @param sim_step - der aktuelle Simulationsschritt als Integerwert
	 */
	void useAllRules(int sim_step, CounterSumAtSimStep* counterSumAtSimStep);

	/*
	 * Gibt zurueck ob in der direkten Umgebung ein Platz frei ist.
	 *
	 * @param x - die X-Position in dessen Umkreis gesucht werden soll
	 * @param y - die Y-Position in dessen Umkreis gesucht werden soll
	 */
	bool KeinPlatzInUmgebung(int x, int y);

	/*
	 * Bewegt alle Kreaturen auf dem aktuellen Spielfeld nach dessen Regeln. Und generiert,
	 * wo moeglich neue Menschen.
	 */
	void moveCreaturesAndSetNewborns(int sim_step);

	/*
	 * Gibt die Position der Kreatur im Array zurck
	 *
	 * @param x - X Position
	 * @param y - Y Position
	 * @param z - Z Position
	 */
	int getPositionInArray(int x, int y, int z);

#endif /* GAMEFIELD_H_ */
