/*
 * generateandset.c
 *
 * Stellt die benötigten Services zur Verfügung, um neue Kreaturen in das Spielfeld zu setzen
 *
 *  Created on: 10.08.2015
 *      Author: Jun-Patrick Raabe & Kolya Feierabend
 */
#include "generateandset.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>


#include "output.h"
#include "config.h"
#include "creature.h"
#include "gamefield.h"
#include "parallel.h"

bool newbornPossible (int x, int y);
void setHuman();
void setVampires();
void setHunter();
void setBlade();
void setNewBorn(int x, int y, int sim_step, CounterSumAtSimStep* counterSumAtSimStep);

int getPopulationForEachRank(int rank, int numprocs, int start_popu_race);

void setNewbornIfPossibile(int x, int y, int sim_step, CounterSumAtSimStep* counterSumAtSimStep){
	if (newbornPossible(x, y)) {
		if (!KeinPlatzInUmgebung(x, y)) {
			setNewBorn(x, y, sim_step, counterSumAtSimStep);
		}
	}
}

void setAllCreatures(){
	setHuman();
	setVampires();
	setHunter();
	setBlade();
}

/*
 * Prueft, ob 2 "für das Kinder kriegen" geeignete Kreaturen auf einem Platz stehen und diese auch das richtige Alter besitzen
 *
 *@param x - die aktuelle X-Postion von der aus geschaut werden soll
 *@param y - die aktuelle Y-Postion von der aus geschaut werden soll
 */
bool newbornPossible (int x, int y) {
	return ((gamefield[getPositionInArray(x, y, 0)].isHuman && gamefield[getPositionInArray(x, y, 1)].isHuman) || (gamefield[getPositionInArray(x, y, 0)].isHuman && gamefield[getPositionInArray(x, y, 1)].isHunter) || (gamefield[getPositionInArray(x, y, 0)].isHunter && gamefield[getPositionInArray(x, y, 1)].isHuman) || (gamefield[getPositionInArray(x, y, 0)].isHunter && gamefield[getPositionInArray(x, y, 1)].isHunter)) &&
			((gamefield[getPositionInArray(x, y, 0)].woman  && !gamefield[getPositionInArray(x, y, 1)].woman) || (!gamefield[getPositionInArray(x, y, 0)].woman && gamefield[getPositionInArray(x, y, 1)].woman)) &&
			((gamefield[getPositionInArray(x, y, 0)].age >= min_childbirth_age) && (gamefield[getPositionInArray(x, y, 0)].age <= max_childbirth_age) && (gamefield[getPositionInArray(x, y, 1)].age >= min_childbirth_age) && (gamefield[getPositionInArray(x, y, 1)].age <= max_childbirth_age));
}


/*
 * Setzt einen Menschen auf eine zufällige Position im Spielfeld - das Alter und Geschlecht werden zufällig ausgewählt
 */
void setHuman(){
	int pos_x, pos_y, humenToSetForRank, allowed_min_x_Pos, allowed_max_x_Pos;

	humenToSetForRank = getPopulationForEachRank(rank, numprocs, start_popu_human);

	allowed_min_x_Pos = getAllowed_min_x_Pos(rank, numprocs);
	allowed_max_x_Pos = getAllowed_max_x_Pos(rank, numprocs, allowed_min_x_Pos);

	for(int i = 1; i <= humenToSetForRank; ++i) {
		bool creatureSet = false;
		do {
			pos_x = allowed_min_x_Pos + rand() % (allowed_max_x_Pos - allowed_min_x_Pos + 1);
			pos_y = rand() % (world_size_y + 1);
			for(int pos_z = 0; pos_z <=1; pos_z++)
			{
				if (!CreatureNotDefault((&gamefield[getPositionInArray(pos_x, pos_y, pos_z)])))
				{
					int age;

					bool age1or2 = rand() % 2;
					bool woman = rand() % 2;
					if (age1or2){
						age = rand() % (min_fighting_age);
					} else {
						age = (rand() % (human_death_age - max_fighting_age)) + max_fighting_age;
					}
					setDefaultToHuman(&gamefield[getPositionInArray(pos_x, pos_y, pos_z)], age, woman, 0, &counterSum[0]);
					creatureSet = true;
					break;
				}
			}
		} while (!creatureSet);
	}
}

/*
 * Setzt einen Vampir auf eine zufällige Position im Spielfeld
 */
void setVampires(){
	int pos_x, pos_y, vampiresToSetForRank, allowed_min_x_Pos, allowed_max_x_Pos;
	vampiresToSetForRank = getPopulationForEachRank(rank, numprocs, start_popu_vampire);

	allowed_min_x_Pos = getAllowed_min_x_Pos(rank, numprocs);
	allowed_max_x_Pos = getAllowed_max_x_Pos(rank, numprocs, allowed_min_x_Pos);

	for(int i = 1; i <= vampiresToSetForRank; ++i) {
		bool creatureSet = false;
		do {
			pos_x = allowed_min_x_Pos + rand() % (allowed_max_x_Pos - allowed_min_x_Pos + 1);
			pos_y = rand() % (world_size_y + 1);
			for(int pos_z = 0; pos_z <=1; pos_z++)
			{
				if (!CreatureNotDefault((&gamefield[getPositionInArray(pos_x, pos_y, pos_z)])))
				{
					int hungerrate = rand() % vampire_max_hunger_rate;
					setDefaultToVampire(&gamefield[getPositionInArray(pos_x, pos_y, pos_z)], hungerrate, 0, &counterSum[0]);
					creatureSet = true;
					break;
				}
			}
		} while (!creatureSet);
	}
}

/*
 * Setzt einen Jaeger auf eine zufällige Position im Spielfeld
 */
void setHunter(){
	int pos_x, pos_y, huntersToSetForRank, allowed_min_x_Pos, allowed_max_x_Pos;

	huntersToSetForRank = getPopulationForEachRank(rank, numprocs, start_popu_hunter);

	allowed_min_x_Pos = getAllowed_min_x_Pos(rank, numprocs);
	allowed_max_x_Pos = getAllowed_max_x_Pos(rank, numprocs, allowed_min_x_Pos);

	for(int i = 1; i <= huntersToSetForRank; ++i) {
		bool creatureSet = false;
		do {
			pos_x = allowed_min_x_Pos + rand() % (allowed_max_x_Pos - allowed_min_x_Pos + 1);
			pos_y = rand() % (world_size_y + 1);
			for(int pos_z = 0; pos_z <=1; pos_z++)
			{

				if (!CreatureNotDefault((&gamefield[getPositionInArray(pos_x, pos_y, pos_z)])))
				{
					bool woman = rand() % 2;
					int age =  min_fighting_age + (rand() % (max_fighting_age - min_fighting_age +1));
					setDefaultToHunter(&gamefield[getPositionInArray(pos_x, pos_y, pos_z)], age, woman, 0, &counterSum[0]);
					creatureSet = true;
					break;
				}
			}
		} while (!creatureSet);
	}
}

/*
 * Setzt den Joker Blade auf eine zufällige Position im Spielfeld
 */
void setBlade(){
	if (rank == 0) {
		int pos_x, pos_y;
		if (blade_available){
			bool creatureSet = false;
			do {
				pos_x = rand() % (world_size_x + 1);
				pos_y = rand() % (world_size_y + 1);
				for(int pos_z = 0; pos_z <=1; pos_z++)
				{
					if (!CreatureNotDefault((&gamefield[getPositionInArray(pos_x, pos_y, pos_z)])))
					{
						setDefaultToBlade(&gamefield[getPositionInArray(pos_x, pos_y, pos_z)], 0, &counterSum[0]);
						creatureSet = true;
						break;
					}
				}
			} while (!creatureSet);
		}
	}
}

/*
 * Setzt ein Neugeborenes mit zufälligem Geschlecht auf eine freie Nachbarposition der Eltern
 *
 * @param x - die X-Position in dessen Umkreis gesucht werden soll
 * @param y - die Y-Position in dessen Umkreis gesucht werden soll
 * @param sim_step - der aktuelle Simulationsschritt als Integerwert
 */
void setNewBorn(int x, int y, int sim_step, CounterSumAtSimStep* counterSumAtSimStep){
	bool setNewborn = false;
	int pos_x, pos_y;
	do {
		//set Newborn
		pos_x = GetNextXPos(x);
		pos_y = GetNextYPos(y);

		for(int place = 0; place < 2; ++place) {
			if (!CreatureNotDefault((&gamefield[getPositionInArray(pos_x, pos_y, place)]))) {
				bool woman = rand() % (2);
				setDefaultToHuman(&gamefield[getPositionInArray(pos_x, pos_y, place)], 0, woman, sim_step, counterSumAtSimStep);
				gamefield[getPositionInArray(pos_x, pos_y, place)].alreadyMoved = true;
				setNewborn = true;
				break;
			}
		}
	} while (!setNewborn);

//	gamefield[creaturePositionInArray(x, y, 0)].alreadyMoved = false;
//	gamefield[creaturePositionInArray(x, y, 1)].alreadyMoved = false;
}

int getAllowed_min_x_Pos(int rank, int numprocs){
	int allowed_min_x_Pos, verschiebung;

	if(rank == 0)
		allowed_min_x_Pos = 0;
	else {
		if ((rank >= (world_size_x+1) % numprocs))
			verschiebung = (world_size_x+1) % numprocs;
		else
			verschiebung = rank;

		allowed_min_x_Pos = (((world_size_x+1) / numprocs) * rank) + verschiebung;
	}

	return allowed_min_x_Pos;
}

int getAllowed_max_x_Pos(int rank, int numprocs, int allowed_min_x_Pos){
	int allowed_max_x_Pos;
	if (rank == (numprocs-1))
		allowed_max_x_Pos = world_size_x;
	else{
		allowed_max_x_Pos = allowed_min_x_Pos + (world_size_x  / numprocs)-1;
		if (rank < ((world_size_x+1) % numprocs))
			allowed_max_x_Pos++;
	}

	return allowed_max_x_Pos;
}
/*
 * Gibt die Bevölkerungsgröße zurück, die der jeweilige Rank berechnen muss
 *
 * @param rank - der Thread
 * @param numprocs - Anzhal der gesamten Threads
 * @param start_popu_race - Bevölkerungsgröße der jeweiligen Rasse
 */
int getPopulationForEachRank(int rank, int numprocs, int start_popu_race){
	int raceToSetForRank;
	raceToSetForRank = start_popu_race/numprocs;
	if(rank < (start_popu_race % numprocs))
		raceToSetForRank++;

	return raceToSetForRank;
}

