/*
 * In vampire.c werden alle Komponenten der Simulation einer Welt mit Vampiren, Jaeger, Menschen
 * und Blade zusammengefuehrt und ausgefuehrt. Im Anschluss werden die Ergebnisse noch ausgegeben.
 *
 *      Author: Jun-Patrick Raabe & Kolya Feierabend
 */
#include <stdio.h> 
#include <stdlib.h>
#include <time.h>
#include <zlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <math.h>
#include <mpi.h>

#include "output.h"
#include "config.h"
#include "gamefield.h"
#include "generateandset.h"
#include "creature.h"
#include "parallel.h"

int main(int argc, char *argv[]) {
	srand((unsigned)time(NULL));

	struct timespec start, end;
	clock_gettime(CLOCK_MONOTONIC, &start);

	init_mpi();
	initResults();
	initGamefield();
	setAllCreatures();
	sendInnerBoarders();

	//Simulationsbegin
	for(int sim_step = 1; sim_step <= sim_duration; ++sim_step)
	{
		MPI_Barrier(MPI_COMM_WORLD);
		counterSum[sim_step] = counterSum[sim_step -1];
		moveCreaturesAndSetNewborns(sim_step);
		MPI_Barrier(MPI_COMM_WORLD);
		useAllRules(sim_step, &counterSum[sim_step]);

		if(picturePrinting){
			sumGamefield();
			if (rank == 0)
				createPicture(sim_step);
		}
	}

	MPI_Barrier(MPI_COMM_WORLD);
	clock_gettime(CLOCK_MONOTONIC, &end);

	sumCounter();

	if (rank == 0) {
		printResultsToConsole(start.tv_sec, end.tv_sec);
		saveResultsToFile();
	}

	MPI_Finalize();

    return 0;
}
