/*
 * output.c
 *
 * Stellt alle Services zur Verfuegung, die benoetigt werden um die Ergebnisse
 * der Simulation aufzuzeichen und auszugeben.
 *
 *  Created on: 07.08.2015
 *      Author: Jun-Patrick Raabe & Kolya Feierabend
 */
#include "output.h"

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "parallel.h"
#include "creature.h"
#include "gamefield.h"
#include "bmp.h"

CounterSumAtSimStep *counterSum;


void initResults(){
	counterSum = (CounterSumAtSimStep *)malloc(sizeof(CounterSumAtSimStep) * (sim_duration+1));
	//Set all Creatures to default
	for (int sim_step = 0; sim_step <= sim_duration; ++sim_step) {
		counterSum[sim_step] = initCounterSumAtSimStep();
	}
}

void printResultsToConsole(int start, int ende){
	printf("\n----------Initialisierung----------\n");
	printf("Verwendete Prozesse:%d\n", numprocs);
	printf("Erzeugte Menschen: %d\n", counterSum[0].humanCounter);
	printf("Erzeugte Vampire: %d\n", counterSum[0].vampireCounter);
	printf("Erzeugte Jaeger: %d\n", counterSum[0].hunterCounter);
	printf("Erzeugte Blades: %d\n", counterSum[0].bladeCounter);
	printf("Erstellte defaultDinger: %d\n", counterSum[0].emptyCounter);
	printf("Summe aller Dinge: %d", counterSum[0].humanCounter + counterSum[0].vampireCounter + counterSum[0].hunterCounter + counterSum[0].bladeCounter + counterSum[0].emptyCounter);
	printf(" = %d\n", (world_size_x +1) * (world_size_y+1) *2);

	printf("\n-------------Resultat-------------\n");
	printf("Ueberlebte Menschen: %d\n", counterSum[sim_duration].humanCounter);
	printf("Ueberlebte Vampire: %d\n", counterSum[sim_duration].vampireCounter);
	printf("Ueberlebte Jaeger: %d\n", counterSum[sim_duration].hunterCounter);
	printf("Ueberlebte Blades: %d\n", counterSum[sim_duration].bladeCounter);
	printf("Ueberlebte defaultDinger: %d\n", counterSum[sim_duration].emptyCounter);
	printf("Summe aller ueberlebten Dinger: %d", counterSum[sim_duration].humanCounter+counterSum[sim_duration].vampireCounter+counterSum[sim_duration].hunterCounter+counterSum[sim_duration].bladeCounter+counterSum[sim_duration].emptyCounter);
	printf(" = %d\n", (world_size_x +1) * (world_size_y+1) *2);
	printf("Sekunden: %d\n", (ende - start));
	printf("Simulationsschritte/Sekunde: %f\n", ((double)sim_duration)  / ((double)(ende - start)) );
}

void saveResultsToFile(){
	FILE *fileHandle;
	fileHandle = fopen("Output.csv", "w");
	if (fileHandle != NULL) {
		fprintf(fileHandle, "#;Menschen;Vampire;Jaeger;Blade;Default\n");
		for (int sim_step = 0; sim_step <= sim_duration; ++sim_step) {
			fprintf(fileHandle, "%d;", sim_step);
			fprintf(fileHandle, "%d;", counterSum[sim_step].humanCounter);
			fprintf(fileHandle, "%d;", counterSum[sim_step].vampireCounter);
			fprintf(fileHandle, "%d;", counterSum[sim_step].hunterCounter);
			fprintf(fileHandle, "%d;", counterSum[sim_step].bladeCounter);
			fprintf(fileHandle, "%d;", counterSum[sim_step].emptyCounter);

			fprintf(fileHandle, "\n");
		}
			fclose(fileHandle);
	} else {
		printf("Fehler: Output-Datei konnte nicht geoeffnet werden\n" );
	}
}

void createPicture(int sim_step){

	char filepath[256];
	sprintf(filepath, "frame%d.bmp", sim_step);


	char *pixel_map;
	pixel_map = malloc(sizeof(char) * (world_size_x+1) * (world_size_y+1) * 3);


	for (int x = 0; x <= world_size_x; x++)
	{
		for (int y = 0; y <= world_size_y; y++)
		{
			Creature creature = gamefield[getPositionInArray(x, y, 0)];

			int index = 3 * (y * (world_size_x + 1) + x);
			pixel_map[index] = 0; // rot
			pixel_map[index + 1] = 0; // grn
			pixel_map[index + 2] = 0; // blau
			if (creature.isVampire)
				pixel_map[index] = 255; // Vampire rot
			else if (creature.isHuman)
				pixel_map[index + 2] = 255; // Menschen sind blau
			else if (creature.isHunter)
				pixel_map[index + 1] = 255; // Jger sind grn
			else if (creature.isBlade)
			{
				pixel_map[index] = 0;
				pixel_map[index + 1] = 0;
				pixel_map[index + 2] = 0;
			}
			else if(!CreatureNotDefault(&creature))
			{
				pixel_map[index] = 255;
				pixel_map[index + 1] = 255;
				pixel_map[index + 2] = 255;
			}

		}
	}
	write_bmp(filepath, world_size_x+1, world_size_y+1, pixel_map);
	free(pixel_map);
}

CounterSumAtSimStep initCounterSumAtSimStep() {
	CounterSumAtSimStep counterSumAtSimStep;
	counterSumAtSimStep.humanCounter = 0;
	counterSumAtSimStep.vampireCounter = 0;
	counterSumAtSimStep.hunterCounter = 0;
	counterSumAtSimStep.bladeCounter = 0;
	counterSumAtSimStep.emptyCounter = 0;
	return counterSumAtSimStep;
}


