/*
 * parallel.c
 *
 *  Created on: 07.09.2015
 *      Author: Kolya
 */
#include <mpi.h>

#include "parallel.h"
#include "gamefield.h"
#include "config.h"
#include "creature.h"
#include "output.h"
#include "generateandset.h"

int rank, numprocs;
int allowed_min_x_Pos, allowed_max_x_Pos;

void init_mpi_datatypes();

void init_mpi() {
	MPI_Init(NULL, NULL);
	MPI_Comm_rank(MPI_COMM_WORLD,&rank);
	MPI_Comm_size(MPI_COMM_WORLD,&numprocs);

	init_mpi_datatypes();

	allowed_min_x_Pos = getAllowed_min_x_Pos(rank, numprocs);
	allowed_max_x_Pos = getAllowed_max_x_Pos(rank, numprocs, allowed_min_x_Pos);
}

void init_mpi_datatypes() {
	//mpi_CounterSumAtSimStep
	MPI_Aint humanCounter_address, vampireCounter_address, hunterCounter_address, bladeCounter_address, emptyCouter_address, address;
	CounterSumAtSimStep counterSumAtSimStep;

	MPI_Get_address(&counterSumAtSimStep, &address);
	MPI_Get_address(&counterSumAtSimStep.humanCounter, &humanCounter_address);
	MPI_Get_address(&counterSumAtSimStep.vampireCounter, &vampireCounter_address);
	MPI_Get_address(&counterSumAtSimStep.hunterCounter, &hunterCounter_address);
	MPI_Get_address(&counterSumAtSimStep.bladeCounter, &bladeCounter_address);
	MPI_Get_address(&counterSumAtSimStep.emptyCounter, &emptyCouter_address);

	int counterSumAtSimStep_blocks[] = {1,1,1,1,1};
	MPI_Aint counterSumAtSimStep_offset[5];
	MPI_Datatype counterSumAtSimStep_types[5] = {MPI_INT, MPI_INT, MPI_INT, MPI_INT, MPI_INT};

	counterSumAtSimStep_offset[0] = humanCounter_address - address;
	counterSumAtSimStep_offset[1] = vampireCounter_address - address;
	counterSumAtSimStep_offset[2] = hunterCounter_address - address;
	counterSumAtSimStep_offset[3] = bladeCounter_address - address;
	counterSumAtSimStep_offset[4] = emptyCouter_address - address;

	MPI_Type_create_struct(5, counterSumAtSimStep_blocks, counterSumAtSimStep_offset, counterSumAtSimStep_types, &mpi_counterSumAtSimStep);
	MPI_Type_commit(&mpi_counterSumAtSimStep);

	//mpi_Creature
	MPI_Aint age_address, woman_address, isHunter_address, isHuman_address, isBlade_address, isVampire_address, alreadyMoved_address, hungerRate_address;
	Creature creature;

	MPI_Get_address(&creature, &address);
	MPI_Get_address(&creature.age, &age_address);
	MPI_Get_address(&creature.woman, &woman_address);
	MPI_Get_address(&creature.isHunter, &isHunter_address);
	MPI_Get_address(&creature.isHuman, &isHuman_address);
	MPI_Get_address(&creature.isBlade, &isBlade_address);
	MPI_Get_address(&creature.isVampire, &isVampire_address);
	MPI_Get_address(&creature.alreadyMoved, &alreadyMoved_address);
	MPI_Get_address(&creature.hungerRate, &hungerRate_address);

	int creature_blocks[] = {1,1,1,1,1,1,1,1};
	MPI_Aint creature_offset[8];
	MPI_Datatype creature_types[8] = {MPI_INT, MPI_INT, MPI_INT, MPI_INT, MPI_INT, MPI_INT, MPI_INT, MPI_INT};

	creature_offset[0] = age_address - address;
	creature_offset[1] = woman_address - address;
	creature_offset[2] = isHunter_address - address;
	creature_offset[3] = isHuman_address - address;
	creature_offset[4] = isBlade_address - address;
	creature_offset[5] = isVampire_address - address;
	creature_offset[6] = alreadyMoved_address - address;
	creature_offset[7] = hungerRate_address - address;

	MPI_Type_create_struct(8, creature_blocks, creature_offset, creature_types, &mpi_creature);
	MPI_Type_commit(&mpi_creature);
}

void sumGamefield() {
	if (rank != 0) {
		MPI_Send(&gamefield[getPositionInArray(getAllowed_min_x_Pos(rank, numprocs), 0, 0)], Y_SIZE*Z_SIZE*((getAllowed_max_x_Pos(rank, numprocs, getAllowed_min_x_Pos(rank, numprocs)) - getAllowed_min_x_Pos(rank, numprocs)) + 1), mpi_creature, 0, rank, MPI_COMM_WORLD);
	}
	if (rank == 0) {
		for (int rank = 1; rank < numprocs; ++rank) {
			MPI_Recv(&gamefield[getPositionInArray(getAllowed_min_x_Pos(rank, numprocs), 0, 0)], Y_SIZE*Z_SIZE*((getAllowed_max_x_Pos(rank, numprocs, getAllowed_min_x_Pos(rank, numprocs)) - getAllowed_min_x_Pos(rank, numprocs)) + 1), mpi_creature, rank, rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
		}
	}
}

void sumCounter() {
	//Send changes of counters to root
	if (rank != 0) {
		for(int sim_step = 0; sim_step <= sim_duration; ++sim_step) {
			MPI_Send(&counterSum[sim_step], 1, mpi_counterSumAtSimStep, 0, rank, MPI_COMM_WORLD);
		}
	}
	if (rank == 0) {
		CounterSumAtSimStep otherCounter;
		for(int sim_step = 0; sim_step <= sim_duration; ++sim_step) {
			for (int rank = 1; rank < numprocs; ++rank) {
				MPI_Recv(&otherCounter, 1, mpi_counterSumAtSimStep, rank, rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
				counterSum[sim_step].humanCounter = counterSum[sim_step].humanCounter + otherCounter.humanCounter;
				counterSum[sim_step].vampireCounter = counterSum[sim_step].vampireCounter + otherCounter.vampireCounter;
				counterSum[sim_step].hunterCounter = counterSum[sim_step].hunterCounter + otherCounter.hunterCounter;
				counterSum[sim_step].bladeCounter = counterSum[sim_step].bladeCounter + otherCounter.bladeCounter;
				counterSum[sim_step].emptyCounter = counterSum[sim_step].emptyCounter - otherCounter.humanCounter - otherCounter.vampireCounter - otherCounter.hunterCounter - otherCounter.bladeCounter;
			}
		}
	}
}

void sendInnerBoarders() {
	if (rank != 0) {
		//receive left boarder
		MPI_Recv(&gamefield[getPositionInArray((allowed_min_x_Pos - 2), 0, 0)], Y_SIZE * Z_SIZE * 2, mpi_creature, (rank-1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	}
	if (rank != (numprocs - 1)) {
		//send right boarder
		MPI_Send(&gamefield[getPositionInArray((allowed_max_x_Pos - 1), 0, 0)], Y_SIZE * Z_SIZE * 2, mpi_creature, (rank + 1), (rank + 1), MPI_COMM_WORLD);
	}
	if (rank != (numprocs - 1)) {
		//receive right boarder
		MPI_Recv(&gamefield[getPositionInArray((allowed_max_x_Pos + 1), 0, 0)], Y_SIZE * Z_SIZE * 2, mpi_creature, (rank+1), rank, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	}
	if (rank != 0) {
		//send left boarder
		MPI_Send(&gamefield[getPositionInArray(allowed_min_x_Pos, 0, 0)], Y_SIZE * Z_SIZE * 2, mpi_creature, (rank - 1), (rank - 1), MPI_COMM_WORLD);
	}
}

